import unittest

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from tugbot import complex_date_parser

#----------------
class Test_complex_date_parser(unittest.TestCase):
    def test_get_dates(self):
        test1_question = "{{complex date|date1=19|precision1=century}}"
        test1_solution = {
                          'date1': {'day':None, 'month':None, 'year':'1900', 'precision':'century' },
                          'date0': {'day':None, 'month':None, 'year':'1900', 'precision':'century' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )
        
        test1_question = "{{complex date|date1=1912}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'1912' }, 
                            'date0': {'day':None, 'month':None, 'year':'1912' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )
        
        test1_question = "{{complex date|date1=10 December 1948}}"
        test1_solution = {
                            'date1': {'day':'10', 'month':'12', 'year':'1948' }, 
                            'date0': {'day':'10', 'month':'12', 'year':'1948' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )  
        
        test1_question = "{{complex date|date1=10 December 1948|date2=3 September 1953}}"
        test1_solution = {
                            'date1': {'day':'10', 'month':'12', 'year':'1948' }, 
                            'date2': {'day':'03', 'month':'09', 'year':'1953' },
                            'date0': {'day':None, 'month':None, 'year':'2000', 'precision':'century' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )
        
        test1_question = "{{complex date|date1=9th century BCE|date2=8th century BCE}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'-900', 'precision':'century' }, 
                            'date2': {'day':None, 'month':None, 'year':'-800', 'precision':'century' },
                            'date0': {'day':None, 'month':None, 'year':'-1000', 'precision':'millennium' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )   
        
        # millennium over 2nd and 1st BCE, but mostly 1
        test1_question = "{{complex date|date1=11th century BCE|date2=8th century BCE}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'-1100', 'precision':'century' }, 
                            'date2': {'day':None, 'month':None, 'year':'-800', 'precision':'century' },
                            'date0': {'day':None, 'month':None, 'year':'-1000', 'precision':'millennium' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )  
        
        # millennium over 2nd and 1st BCE, but mostly 2
        test1_question = "{{complex date|date1=15th century BCE|date2=8th century BCE}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'-1500', 'precision':'century' }, 
                            'date2': {'day':None, 'month':None, 'year':'-800', 'precision':'century' },
                            'date0': {'day':None, 'month':None, 'year':'-2000', 'precision':'millennium' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )
        
        # millennium over 2nd and 1st BCE, but mostly 2
        test1_question = "{{complex date|date1=15|precision1=century|era1=BCE|date2=8|precision2=century|era2=BCE}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'-1500', 'precision':'century' }, 
                            'date2': {'day':None, 'month':None, 'year':'-800', 'precision':'century' },
                            'date0': {'day':None, 'month':None, 'year':'-2000', 'precision':'millennium' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )        
        
        # real-file test from "Category:Armed warrior goddess-AO 20160"
        test1_question = "{{complex date|circa|date1=1500|adj1=|precision1=|era1=BCE|date2=1250|adj2=|precision2=|era2=BCE}}"
        test1_solution = {
                            'date1': {'day':None, 'month':None, 'year':'-1500' }, 
                            'date2': {'day':None, 'month':None, 'year':'-1250' },
                            'date0': {'day':None, 'month':None, 'year':'-2000', 'precision':'millennium' }
                         }
        date = complex_date_parser.Complex_date_parser(test1_question)
        self.assertEqual( date.dates, test1_solution )        
        
        
        
        #test1_question = "{{complex date|date1=1st millennium BCE|date2=2nd century}}"
        #test1_solution = {'date1': {'day':None, 'month':None, 'year':'-1000', 'precision':'millennium' }, 'date2': {'day':None, 'month':None, 'year':'200', 'precision':'century' }}
        #date = complex_date_parser.Complex_date_parser(test1_question)
        #self.assertEqual( date.dates, test1_solution ) 
        
        
