import unittest

import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from tugbot import date_parser
#----------------
class Test_date_parser(unittest.TestCase):
    def test_parse_year_intervals(self):
        self.assertEqual( date_parser.parse_year_intervals("1901-1902"), "1901" )
        self.assertEqual( date_parser.parse_year_intervals("765-1032"), "765" )
    
    def test_parse_year_month_day(self):
        solution_1912_12_04 = ['1912', '12', '04']
        solution_912_09_04  = ['912', '09', '04']
        self.assertEqual( date_parser.parse_year_month_day("1912-12-04"), solution_1912_12_04 ) 
        self.assertEqual( date_parser.parse_year_month_day("1912 12 04"), solution_1912_12_04 )         
        self.assertEqual( date_parser.parse_year_month_day("1912.12.04"), solution_1912_12_04 )             
        self.assertEqual( date_parser.parse_year_month_day("1912,12,04"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("1912;12;04"), solution_1912_12_04 )   
        self.assertEqual( date_parser.parse_year_month_day("1912:12:04"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("1912-12-4"),  solution_1912_12_04 ) 
        self.assertEqual( date_parser.parse_year_month_day("1912 12 4"),  solution_1912_12_04 )         
        self.assertEqual( date_parser.parse_year_month_day("1912.12.4"),  solution_1912_12_04 )             
        self.assertEqual( date_parser.parse_year_month_day("1912,12,4"),  solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("1912;12;4"),  solution_1912_12_04 )   
        self.assertEqual( date_parser.parse_year_month_day("1912:12:4"),  solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("912:09:04"),  solution_912_09_04  )
        self.assertEqual( date_parser.parse_year_month_day("912:9:4"),    solution_912_09_04  )
        self.assertIsNone( date_parser.parse_year_month_day("912:912:4") )
        self.assertIsNone( date_parser.parse_year_month_day("912:13:4") )
        self.assertIsNone( date_parser.parse_year_month_day("912:12:32") )
        self.assertIsNone( date_parser.parse_year_month_day("912:11:31") )
        # leap years
        self.assertEqual ( date_parser.parse_year_month_day("2020-02-29"), ['2020', '02', '29'] ) 
        self.assertIsNone( date_parser.parse_year_month_day("2021-02-29") )
        # named months
        self.assertEqual( date_parser.parse_year_month_day("1912 December 04"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("1912 dec 04"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_year_month_day("1912 April 04"), ['1912', '04', '04'] )
        self.assertEqual( date_parser.parse_year_month_day("1912 apr 04"), ['1912', '04', '04'] )
        
    def test_parse_year_month(self):
        solution_1912_12 = ['1912', '12']
        solution_912_09  = ['912', '09']
        self.assertEqual( date_parser.parse_year_month("1912-12"), solution_1912_12 ) 
        self.assertEqual( date_parser.parse_year_month("1912 12"), solution_1912_12 )         
        self.assertEqual( date_parser.parse_year_month("1912.12"), solution_1912_12 )             
        self.assertEqual( date_parser.parse_year_month("1912,12"), solution_1912_12 )
        self.assertEqual( date_parser.parse_year_month("1912;12"), solution_1912_12 )   
        self.assertEqual( date_parser.parse_year_month("1912:12"), solution_1912_12 )
        self.assertEqual( date_parser.parse_year_month("1912:12:04"), solution_1912_12 )
        self.assertEqual( date_parser.parse_year_month("912:09"),  solution_912_09  )
        self.assertEqual( date_parser.parse_year_month("912:9"),   solution_912_09  )
        self.assertIsNone( date_parser.parse_year_month("912:13") )
        
    def test_parse_day_month_year(self):
        solution_1912_12_04 = ['1912', '12', '04']
        solution_912_09_04  = ['912', '09', '04']
        self.assertEqual( date_parser.parse_day_month_year("04-12-1912"), solution_1912_12_04 ) 
        self.assertEqual( date_parser.parse_day_month_year("04 12 1912"), solution_1912_12_04 )         
        self.assertEqual( date_parser.parse_day_month_year("04.12.1912"), solution_1912_12_04 )             
        self.assertEqual( date_parser.parse_day_month_year("04,12,1912"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_day_month_year("04;12;1912"), solution_1912_12_04 )   
        self.assertEqual( date_parser.parse_day_month_year("04:12:1912"), solution_1912_12_04 )
        
        self.assertEqual( date_parser.parse_day_month_year("4-12-1912"), solution_1912_12_04 ) 
        self.assertEqual( date_parser.parse_day_month_year("4 12 1912"), solution_1912_12_04 )         
        self.assertEqual( date_parser.parse_day_month_year("4.12.1912"), solution_1912_12_04 )             
        self.assertEqual( date_parser.parse_day_month_year("4,12,1912"), solution_1912_12_04 )
        self.assertEqual( date_parser.parse_day_month_year("4;12;1912"), solution_1912_12_04 )   
        self.assertEqual( date_parser.parse_day_month_year("4:12:1912"), solution_1912_12_04 )
        
        self.assertEqual( date_parser.parse_day_month_year("04 09 912"),  solution_912_09_04  )
        self.assertEqual( date_parser.parse_day_month_year("4-9-912"),    solution_912_09_04  )
        self.assertIsNone( date_parser.parse_day_month_year("4:912:912") )
        self.assertIsNone( date_parser.parse_day_month_year("4 13 912") )
        self.assertIsNone( date_parser.parse_day_month_year("32 12 912") )
        self.assertIsNone( date_parser.parse_day_month_year("31 11 912") )
        
    def test_parse_month_year(self):
        solution_1912_12 = ['1912', '12']
        solution_912_09  = ['912', '09']
        self.assertEqual( date_parser.parse_month_year("12-1912"), solution_1912_12 ) 
        self.assertEqual( date_parser.parse_month_year("12 1912"), solution_1912_12 )         
        self.assertEqual( date_parser.parse_month_year("12.1912"), solution_1912_12 )             
        self.assertEqual( date_parser.parse_month_year("12,1912"), solution_1912_12 )
        self.assertEqual( date_parser.parse_month_year("12;1912"), solution_1912_12 )   
        self.assertEqual( date_parser.parse_month_year("12:1912"), solution_1912_12 )
    
        self.assertIsNone( date_parser.parse_month_year("13 912") )
        
    def test_get_data(self): # actually using this to test the constructor more than anything else.
        test_date = date_parser.Date_parser("18 century")
        self.assertEqual( test_date.get_data(), {"year":"1800", "month":None, "day":None, "precision": "century"} )
        test_date = date_parser.Date_parser("18th century")
        self.assertEqual( test_date.get_data(), {"year":"1800", "month":None, "day":None, "precision": "century"} )            
        test_date = date_parser.Date_parser("2nd millennium")
        self.assertEqual( test_date.get_data(), {"year":"2000", "month":None, "day":None, "precision": "millennium"} )            
        test_date = date_parser.Date_parser("2nd millennium BCE")
        self.assertEqual( test_date.get_data(), {"year":"-2000", "month":None, "day":None, "precision": "millennium"} )    
        
if __name__ == '__main__':
    unittest.main()
    
    
    
    
    
    
    
