import mwparserfromhell
import re
import time
import datetime
import calendar
#import category_texts

#==========================================
def string_to_int_month(date_text):
    # This is not very robust.
    date_text = date_text.lower()
    months     = {"january"  :"01",
                  "february" :"02",
                  "march"    :"03",
                  "april"    :"04",
                  "may"      :"05",
                  "june"     :"06",
                  "july"     :"07",
                  "august"   :"08",
                  "september":"09",                                    
                  "october"  :"10",
                  "november" :"11",
                  "december" :"12",
                  }
    month_abrrs = { "jan"      :"01",
                    "feb"      :"02",
                    "mar"      :"03",
                    "apr"      :"04",
                    "jun"      :"06",
                    "jul"      :"07",
                    "aug"      :"08",
                    "sep"      :"09",
                    "oct"      :"10",
                    "nov"      :"11",
                    "dec"      :"12"
                }
    for month in months:
        if month in date_text:
            date_text = date_text.replace(month, months[month])
    for month in month_abrrs:
        if month in date_text:
            date_text = date_text.replace(month, month_abrrs[month])        
    return date_text

def validate_date_vector(date_vector):
    # prevents invalid dates from being returned.
    _year, _month, _day = 0, 0, 0
    if (len(date_vector) == 3): 
        _year, _month, _day = [int(s) for s in date_vector]
        if _day > 31 or _day < 1: return None
        if _month > 12 or _month < 1: return None
        number_of_days_dict = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
        # bissextile years
        if calendar.isleap(_year): number_of_days_dict[1] = 29
        if _day > number_of_days_dict[_month-1]: return None
    elif (len(date_vector) == 2): 
        _year, _month = [int(s) for s in date_vector]
        if _month > 12 or _month < 1: return None
    return date_vector

def parse_year_intervals(date_text):
    # Two years such as "1901-1902", or "783-1034": return only the first number.
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(\d{3,4})[- ,.;:](\d{3,4})', date_text)
    if matchObj: 
        _year   = matchObj.group(1)
        return _year 
    return None

def parse_year_month_day(date_text):
    # When we have the day (I assume this will not happen to years before 100 AD)
    # "1912-12-04", "1912 12 04", "1912,12,04", "1912.12.04", "1912-12-4"
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(\d{3,4})[- ,.;:](\d{1,2})[- ,.;:](\d{1,2})', date_text)
    if matchObj: 
        _year   = matchObj.group(1)
        _month  = matchObj.group(2)
        _day    = matchObj.group(3)
        return validate_date_vector( [_year, format(int(_month), '02d'), format(int(_day), '02d')] )
    return None
    
def parse_year_month(date_text):
    # When we have the day (I assume this will not happen to years before 100 AD)
    # "1912-12", "1912 12", "1912,12", "1912.12", "1912-9"
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(\d{3,4})[- ,.;:](\d{1,2})', date_text)
    if matchObj: 
        _year   = matchObj.group(1)
        _month  = matchObj.group(2)
        return validate_date_vector( [_year, format(int(_month), '02d')] )
    return None
    
    
def parse_day_month_year(date_text):
    # When we have the day (I assume this will not happen to years before 100 AD)
    # "04-12-1912", "04 12 1912", "04, 12, 1912", "02.12.1912", "4-12-1912"
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(\d{1,2})[- ,.;:](\d{1,2})[- ,.;:](\d{3,4})', date_text)
    if matchObj: 
        _year   = matchObj.group(3)
        _month  = matchObj.group(2)
        _day    = matchObj.group(1)
        return validate_date_vector( [_year, format(int(_month), '02d'), format(int(_day), '02d')] )
    return None
    
def parse_month_year(date_text):    
    # When we have the day (I assume this will not happen to years before 100 AD)
    # "1912-12", "1912 12", "1912,12", "1912.12", "1912-9"
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(\d{1,2})[- ,.;:](\d{3,4})', date_text)
    if matchObj: 
        _year   = matchObj.group(2)
        _month  = matchObj.group(1)
        return validate_date_vector( [_year, format(int(_month), '02d')] )
    return None

def parse_general_date(date_text):    
    # Last-recourse parser: simply exctracts a numeral from the given string.
    # "18th century" -> "18"
    # "-2 millennium" -> "-2"
    date_text = string_to_int_month(date_text)
    matchObj = re.search( r'(-?)(\d+)', date_text)
    if matchObj: 
        #print (date_text)
        #print ("matchObj.group(1)", matchObj.group(1))
        #print ("matchObj.group(2)", matchObj.group(2))
        return matchObj.group(1)+matchObj.group(2)
    return None

#==========================================
# A Date_parser will take a string such as
# "2012-08-05", "5 12 1956", "18 century", "2 millennium BCE"
# and translate into a three-component vector representing the 
# year, month and day.

class Date_parser:
    def __init__(self, wikicode):
        date_text = wikicode.strip().lower()
        self.fields = {"year":None, "month":None, "day":None}
        
        # Attempting to parse dates precise to the day or month
        if parse_year_intervals(date_text): # 1901-1902
            self.fields["year"]  = parse_year_intervals(date_text)
        elif parse_year_month_day(date_text): # 1948-12-10
            year, month, day = parse_year_month_day(date_text)
            self.fields["year"]  = year
            self.fields["month"] = month
            self.fields["day"]   = day
        elif parse_year_month(date_text): # 1948-12
            year, month = parse_year_month_day(date_text)
            self.fields["year"]  = year
            self.fields["month"] = month
        elif parse_day_month_year(date_text): # 10-12-1948
            year, month, day = parse_day_month_year(date_text)
            self.fields["year"]  = year
            self.fields["month"] = month
            self.fields["day"]   = day
        elif parse_month_year(date_text): # 12-1948 
            year, month = parse_month_year(date_text)
            self.fields["year"]  = year
            self.fields["month"] = month
        else:
            self.fields["year"]  = parse_general_date(date_text) # 1948, 18th century, 2nd millennium BCE 
            
        # Sets precision for dates precise up to the year, century or millennium
        if self.fields["year"] and ("century" in date_text):
            self.fields["year"]      = 100*int(self.fields["year"])
            self.fields["precision"] = "century"
        if self.fields["year"] and ("millennium" in date_text):
            self.fields["year"]      = 1000*int(self.fields["year"])
            self.fields["precision"] = "millennium"
        if self.fields["year"] and ( "bc" in date_text ):
            self.fields["year"]      = -int(self.fields["year"])
        self.fields["year"] = str( self.fields["year"] )
        
        # Sets "circa"
        if re.search( r'[Cc]irca', date_text): 
            self.fields["adj"] = "circa"
            
#...............                
    def get_data(self):
        return self.fields

    @property
    def day(self):
        if "day" in self.fields:
            return self.fields["day"]
        return None
    
    @property
    def month(self):
        if "month" in self.fields:
            return self.fields["month"]
        return None
    
    @property
    def year(self):
        if "year" in self.fields:
            return self.fields["year"]
        return None
    
    @property
    def precision(self):
        if "precision" in self.fields:
            return self.fields["precision"]
        return "year"
    
    @property
    def is_BCE(self):
        if "is_BCE" in self.fields:
            return self.fields["is_BCE"]
        return False

    def ensure_BCE(self):
        if self.fields["year"]:
            int_year = int(self.fields["year"])
            if int_year > 0: self.fields["year"] = str( -int_year )

        
        
        
        
        
        
        
