import mwparserfromhell

_length_units_list = ['m', 'cm', 'mm', 'mi', 'ft', 'yd', 'in']
_mass_units_list   = ['g', 'kg', 'lb']

#==========================================                                
def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

#==========================================                                
def parse_size(size_text):
    fields = {}
    wikicode_parsed = mwparserfromhell.parse(size_text)
    templates = wikicode_parsed.filter_templates()
        
    for template in templates:
        if template.name.matches("size") or template.name.matches("weight"):
            
            #These two are used when we get a headerless Size template like {{Size|cm|12.1234|98.1234|prec=1}}
            about_mass = False # Default is to be about length
            dimensions_buffer = []
            
            for i in template.params:
                #if i.split("="):
                if ("=") in i:
                    key   = i.split("=", 1)[0].strip()
                    if key == "weight": key = "mass" 
                    value = None
                    if (len(i.split("="))>1): value = i.split("=", 1)[1].strip()
                    if not value or value=="" : continue
                    if value in _length_units_list: fields["length_unit"] = value
                    elif value in _mass_units_list: fields["mass_unit"]   = value
                    else: fields[key] = value
                else:
                    # In case we get a headerless Size template like {{Size|cm|12.1234|98.1234|prec=1}}
                    # the unit will be in front; we use it to detect if we are talking about mass or length.
                    # In case several figures are given, the largest if "length", then "width", and then "thickness"

                    if i in _length_units_list: 
                        fields["length_unit"] = i
                    elif i in _mass_units_list: 
                        fields["mass_unit"]   = i
                        about_mass = True
                    else:
                        if is_number(str(i)): dimensions_buffer.append(i)
                        
            dimensions_buffer = sorted(dimensions_buffer, reverse=True)
            if about_mass:
                if len(dimensions_buffer) == 1: fields["mass"] = dimensions_buffer[0]
            else:
                if len(dimensions_buffer) > 0: fields["length"]    = dimensions_buffer[0]
                if len(dimensions_buffer) > 1: fields["width"]     = dimensions_buffer[1]
                if len(dimensions_buffer) > 2: fields["thickness"] = dimensions_buffer[2]
    return fields

#==========================================

class Dimensions_parser:
    def __init__(self, wikicode):
        self.fields = {}
        if not wikicode: return None
    
        wikicode = wikicode.lower()
        
        if ("{{size" in wikicode): self.fields = parse_size( wikicode )
#---------------------               
    def get_data(self):
        return self.fields
    
    @property
    def length_unit(self):
        if ("length_unit") in self.fields: return self.fields["length_unit"]
        return None
    
    @property
    def mass_unit(self):
        if ("mass_unit") in self.fields: return self.fields["mass_unit"]
        return None    
    
    @property
    def height(self):
        if ("height") in self.fields: return self.fields["height"]
        return None
    
    @property
    def width(self):
        if ("width") in self.fields: return self.fields["width"]
        return None
    
    @property
    def depth(self):
        if ("depth") in self.fields: return self.fields["depth"]
        return None
    
    @property
    def length(self):
        if ("length") in self.fields: return self.fields["length"]
        return None
    
    @property
    def diameter(self):
        if ("diameter") in self.fields: return self.fields["diameter"]
        return None
    
    @property
    def thickness(self):
        if ("thickness") in self.fields: return self.fields["thickness"]
        return None
    
    @property
    def mass(self):
        if ("mass") in self.fields: return self.fields["mass"]
        return None
    
    





        
