import mwparserfromhell
from tugbot import date_parser
from tugbot import complex_date_parser


#==========================================                                
_provenance_Q_numbers = {
                            "commission"    : "Q760089",
                            "in collection" : "Q2668072",
                            "acquisition"   : "Q22340494",
                            "purchase"      : "Q1369832",
                            "purchased"     : "Q1369832",
                            "gift"          : "Q707482",
                            "exchange"      : "Q23009552",
                            "inheritance"   : "Q200303",
                            "bequest"       : "Q211557",
                            "sale"          : "Q194189",
                            "anonymous sale": "Q48692894",
                            "nationalization" : "Q178564",
                            "conveyance"    : "Q759815",
                            "transfer"      : "Q315364",
                            "loan"          : "Q716894",
                            "theft"         : "Q2727213",
                            "restitution"   : "Q1156800",
                            "discovery"     : "Q753297",
                            "destruction"   : "Q17781833",
                            "excavation"    : "Q959782",    
                            "excavated"     : "Q959782"
                        }



#==========================================                                
def parse_provenance(provenance_text):
    fields = {}
    wikicode_parsed = mwparserfromhell.parse(provenance_text)
    templates = wikicode_parsed.filter_templates()
        
    for template in templates:
        if template.name.matches("provenanceevent"):
            medium_counter=0
            key   = None
            value = None
            for i in template.params:
                if ("=") in i:
                    key   = i.split("=", 1)[0].strip()
                    #if "and" in key:
                        #key = "medium"+str(medium_counter)
                        #medium_counter += 1
                    value = None
                    if (len(i.split("="))>1): value = i.split("=", 1)[1].strip()
                    if not value or value=="" : continue
                    fields[key] = value
                else:
                    key = "medium"+str(medium_counter)
                    medium_counter += 1
                    value = i
                fields[key] = value
    return fields

#==========================================
class Provenance_parser:
    def __init__(self, wikicode):
        self.fields = {}
        if not wikicode: return None
    
        wikicode = wikicode.lower()
        if ("{{provenanceevent" in wikicode): self.fields = parse_provenance( wikicode )
#---------------------               
    def get_data(self):
        return self.fields
    
    @property
    def type(self):
        if "type" in self.fields: return self.fields["type"]
        return None
    
    @property
    def type_q_number(self):
        if self.type and self.type in _provenance_Q_numbers:
            return _provenance_Q_numbers[self.type]
        return None

    @property
    def date(self):
        if ("date" not in self.fields) and ("time" not in self.fields):
            return None
        
        date_text = None
        if "date" in self.fields: date_text = self.fields["date"]
        elif "time" in self.fields: date_text =  self.fields["time"]
        if complex_date_parser.is_complex_date(date_text):
            date = complex_date_parser.Complex_date_parser(date_text)
            date_text = date.dates
        date = date_parser.Date_parser(date_text)
        date_text =  date.get_data()
    
        return str(date_text)
    
    @property
    def old_owner (self):
        if "newowner" in self.fields: return self.fields["newowner"]
        return None
        
    @property
    def new_owner(self):
        if "oldowner" in self.fields: return self.fields["oldowner"]
        return None
    
    
    
    
    
    
    
    
    
    
    
